#include "esp_camera.h"
#include <WiFi.h>

//
// WARNING!!! Make sure that you have either selected ESP32 Wrover Module,
//            or another board which has PSRAM enabled
//

// Select camera model
//#define CAMERA_MODEL_WROVER_KIT
//#define CAMERA_MODEL_ESP_EYE
//#define CAMERA_MODEL_M5STACK_PSRAM
//#define CAMERA_MODEL_M5STACK_WIDE
#define CAMERA_MODEL_AI_THINKER

#include "camera_pins.h"

const char *ssid = "x.factory";
const char *password = "make0314";

void startCameraServer();

//#add 来自app_httpd.cpp
bool giveFood();

//------ 时间的相关设置
#include "time.h"
const char *ntpServer = "pool.ntp.org";
const long gmtOffset_sec = 28800; //东八区时间
const int daylightOffset_sec = 0;

//---------#add 设置闪光灯引脚为4------------
#define LED_PIN 4

//--------#add 时间检测----------------
bool isCheck = false;
int start_int = 1230;
int start_int2 = 1231;

//--------#add 伺服电机----------------
#include <ESP32Servo.h>
Servo myservo; // create servo object to control a servo
// 16 servo objects can be created on the ESP32
int pos = 0; // variable to store the servo position
// Recommended PWM GPIO pins on the ESP32 include 2,4,12-19,21-23,25-27,32-33
int servoPin = 2;

void printLocalTime()
{
  struct tm timeinfo;
  if (!getLocalTime(&timeinfo))
  {
    Serial.println("Failed to obtain time");
    return;
  }
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
}
//--------#add 获取时间整数----------------
int getTimeInt()
{
  struct tm timeinfo;
  //  int time_int;
  if (!getLocalTime(&timeinfo))
  {
    Serial.println("Failed to obtain time");
    return 0;
  }
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
  int hour_int = timeinfo.tm_hour;
  int min_int = timeinfo.tm_min;
  String time_string = int2String(hour_int) + int2String(min_int);
  int time_int = time_string.toInt();
  Serial.println("time_int: " + String(time_int));
  return time_int;
}
//--------#add 将小于10的整数加0，并变为字符串----------------
String int2String(int the_time_int)
{
  String the_time_string;
  if (the_time_int < 10)
  {
    the_time_string = "0" + String(the_time_int);
  }
  else
  {
    the_time_string = String(the_time_int);
  }
  return the_time_string;
}

void setup()
{
  Serial.begin(115200);
  Serial.setDebugOutput(true);
  Serial.println();

  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  config.pixel_format = PIXFORMAT_JPEG;
  //init with high specs to pre-allocate larger buffers
  if (psramFound())
  {
    config.frame_size = FRAMESIZE_UXGA;
    config.jpeg_quality = 10;
    config.fb_count = 2;
  }
  else
  {
    config.frame_size = FRAMESIZE_SVGA;
    config.jpeg_quality = 12;
    config.fb_count = 1;
  }

#if defined(CAMERA_MODEL_ESP_EYE)
  pinMode(13, INPUT_PULLUP);
  pinMode(14, INPUT_PULLUP);
#endif

  // camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK)
  {
    Serial.printf("Camera init failed with error 0x%x", err);
    return;
  }

  sensor_t *s = esp_camera_sensor_get();
  //initial sensors are flipped vertically and colors are a bit saturated
  if (s->id.PID == OV3660_PID)
  {
    s->set_vflip(s, 1);       //flip it back
    s->set_brightness(s, 1);  //up the blightness just a bit
    s->set_saturation(s, -2); //lower the saturation
  }
  //drop down frame size for higher initial frame rate
  s->set_framesize(s, FRAMESIZE_QVGA);

#if defined(CAMERA_MODEL_M5STACK_WIDE)
  s->set_vflip(s, 1);
  s->set_hmirror(s, 1);
#endif

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");

  startCameraServer();

  Serial.print("Camera Ready! Use 'http://");
  Serial.print(WiFi.localIP());
  Serial.println("' to connect");

  //# add 时间的相关设置
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  printLocalTime();

  //-------#add 设置闪光灯灯引脚 --------
  pinMode(LED_PIN, OUTPUT);

  //--------#add 伺服电机----------------
  myservo.setPeriodHertz(50);           // standard 50 hz servo
  myservo.attach(servoPin, 1000, 2000); // attaches the servo on pin 18 to the servo object
}

void loop()
{
  // put your main code here, to run repeatedly:

  //  printLocalTime();
  int time_int = getTimeInt();
  //---------根据是否到达时间，进行处理--------
  if (time_int == start_int || time_int == start_int2)
  {
    //---------根据之前是否执行任务，进行处理--------
    if (isCheck)
    {
      Serial.println("has check");
    }
    else
    {
      Serial.println("light on");
      giveFoodNow();
    }
  }
  else
  {
    isCheck = false;
    digitalWrite(LED_PIN, LOW);
    Serial.println("light off");
  }

  bool to_give = giveFood();
  if (to_give)
  {
    Serial.println("light on");
    giveFoodNow();
  }

  delay(1000);
}

void giveFoodNow()
{
  digitalWrite(LED_PIN, HIGH);
  isCheck = true;
  //--------------电机转动----------------v
  for (pos = 0; pos <= 180; pos += 1)
  { // goes from 0 degrees to 180 degrees
    // in steps of 1 degree
    myservo.write(pos); // tell servo to go to position in variable 'pos'
    delay(15);          // waits 15ms for the servo to reach the position
  }
  for (pos = 180; pos >= 0; pos -= 1)
  {                     // goes from 180 degrees to 0 degrees
    myservo.write(pos); // tell servo to go to position in variable 'pos'
    delay(15);          // waits 15ms for the servo to reach the position
  }
  digitalWrite(LED_PIN, LOW);
  //--------------电机转动----------------^
}
